import functools
import logging
import re
import uuid

from typing import List, Dict, Any, Optional, Callable
from contextlib import contextmanager

import textfsm
from ntc_templates.parse import parse_output

from ppa import ui

from . import exceptions
from .records import CiscoSwitch


logger = logging.getLogger(__name__)


def session(func: Callable) -> Callable:
    """
    This decorator unpacks the cisco_switch dictionary and makes sure nothing
    unexpected is passed into the ssh_plugin_driver.client.run_list function.

    If a cisco_switch dictionary is defined in-line in the YAML, the validation
    endpoint will catch any incorrect keys before the task is built.

    However, if the dictionary comes from a vault and is supplied in the YAML
    using load, it will not be validated before the task is built. This can
    trigger exceptions lower down if any unexpected keys are supplied, or if
    any required keys are missing from the dictionary.
    """
    required_keys = list(CiscoSwitch.__annotations__.keys())

    @functools.wraps(func)
    def _filter_connection_details(cisco_switch, *args, **kwargs):
        supplied_keys = list(cisco_switch.keys())

        if missing_keys := [key for key in required_keys if key not in supplied_keys]:
            message = "The following keys are missing from the 'cisco_switch' parameter: '{}'.".format(", ".join(missing_keys))
            ui.output_error(message)
            raise exceptions.ConnectionDetailsError(message)

        sanitised_details = {key: value for key, value in cisco_switch.items() if key in required_keys}
        logger.debug(
            "filtered the following keys out of the cisco_switch dictionary: '{}'".format(
                ", ".join([key for key in supplied_keys if key not in required_keys])
            )
        )

        return func(sanitised_details, *args, **kwargs)

    return _filter_connection_details


@contextmanager
def optional_progress_bar(start_text: str, end_text: str, failure_text: str, display: bool):
    element_id = str(uuid.uuid4())
    raised = False
    try:
        if display:
            ui.output_progress(start_text, percent=None, element_id=element_id)
        yield
    except Exception as e:
        raised = True
        if display:
            ui.output_progress(failure_text, percent=0, element_id=element_id)
        raise e
    finally:
        if all([display, not raised]):
            ui.output_progress(end_text, percent=100, element_id=element_id)


def set_terminal_size(commands: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    # Add terminal size commands to the start of the command list.
    commands.insert(0, {"command": "terminal length 0"})
    commands.insert(0, {"command": "terminal width 0"})
    return commands


def with_enable_mode(commands: List[Dict[str, Any]], enable_password: str) -> List[Dict[str, Any]]:
    # Add the enable mode enter & exit pre & post-ambles to the command list.
    commands.insert(0, {"command": "enable", "questions": [{"prompt": r"^Password:\s+", "answer": enable_password}]})
    commands.append({"command": "disable"})
    return commands


def configure_object(object_type: str, object_name: str, commands: List[Dict[str, Any]], enable_password: Optional[str]):
    # Add the configure terminal & configure object pre & post-ambles to the command list.
    commands.insert(0, {"command": f"{object_type} {object_name}"})
    commands.insert(0, {"command": "configure terminal", "success": ".*Enter configuration commands.*"})
    commands.append({"command": "end"})
    commands = with_enable_mode(commands, enable_password) if enable_password else commands
    logger.debug(f"generated command list: {commands}")
    return commands


def check_command_responses(commands_responses: List[tuple], error_messages: Optional[List[str]] = None) -> None:
    standard_errors = ["Invalid input", "Incomplete command"]
    specific_errors = error_messages or []

    for command, response in commands_responses:
        response_lower = response.lower()

        # General command errors require more context in the exception message
        if any([error.lower() in response_lower for error in standard_errors]):
            message = f"Commmand '{command}' failed with output '{response}'."
            ui.output_error(message)
            raise exceptions.CommandError(message)

        # Specific errors can be supplied by individual actions and require less context
        for error in specific_errors:
            if error.lower() in response_lower:
                ui.output_error(error)
                raise exceptions.CommandError(error)


def parse_command_output(command: str, output: str) -> List[Dict[str, Any]]:
    try:
        if parsed_objects := parse_output(platform="cisco_ios", command=command, data=output):
            return parsed_objects
        else:
            # The parse_output function doesn't always raise exceptions if the input format is unexpected. At this
            # point we should have some rows of objects, so if nothing is generated, there's an issue.
            logger.error("no objects were generated from the following output:")
            logger.error(repr(output))
            raise exceptions.OutputParsingError("Could not parse the command output. Please view the logs for more information.")
    except textfsm.parser.TextFSMError as e:
        # Properly bad input will raise an underlying textfsm exception if we're lucky.
        #
        # Example:
        #     TextFSMError: State Error raised. Rule Line: 140. Input Line: example bad input line
        logger.error("the following command output failed to parse:")
        logger.error(repr(output))
        if match := re.search(r".*Input\sLine:(.*)$", str(e)):
            message = f"Failed to parse '{command}' output. Bad line in response: '{match.group(1)}'"
            logger.info(message)
            raise exceptions.OutputParsingError(message)
        raise e
