from typing import TypedDict, Union, List


class CiscoSwitch(TypedDict):
    r"""
- Contains SSH connection details for an IOS switch

- It is used as an _input_ to many of the actions in this plugin

- The example below shows the dictionary structure in YAML

!!! warning "Supplying Credentials"

    You should ___always___ use a PPA Vault integration to provide credentials to a plugin action.

!!! example

    ```yaml linenums="1"
    cisco_switch:
      address: 1.2.3.4
      port: 22
      username: [username]
      password: [password]
      base_prompt: >
        ^\w+\-.*([>#]|\(.*\)[>#])
    ```

!!! danger "Required Keys"

    __address__: The target IP or DNS address.

    __port__: The target SSH port.

    __username__: Username for authentication.

    __password__: Password for authentication.

    __base_prompt__: A regular expression representing the SSH command prompt.
    """
    address: str
    port: Union[str, int]
    username: str
    password: str
    base_prompt: str


class VLAN(TypedDict):
    """
- Contains switch VLAN properties

!!! info "All VLAN Keys"

    __name__: the VLAN name

    __status__: the current VLAN status

    __interfaces__: a list of interface ports currently assigned to the VLAN

!!! example "VLAN YAML Structure"

    ```yaml linenums="1"
      name: VoIP
      status: active
      interfaces:
        - Gi02
        - Gi03
        - Gi04
    ```

    """

    name: str
    status: Union[str, int]
    interfaces: List[str]


class Interface(TypedDict):
    """
- Contains switch interface properties

!!! info "All Interface Keys"

    __port:__ the interface port

    __name:__ the interface name

    __status:__ the current interface status

    __vlan:__ the ID of the VLAN the interface is currently assigned to

    __duplex:__ the interface duplicity

    __speed:__ the interface speed

    __type:__ the type of the interface

!!! example "Interface YAML Structure"

    ```yaml linenums="1"
      port: Gi0/3
      name: VCenter
      status: connect
      vlan: 200
      duplex: a-full
      speed: a-1000
      type: 10/100/1000BaseTX
    ```

    """

    port: str
    name: str
    status: Union[str, int]
    vlan: str
    duplex: str
    speed: str
    type: str
