"""
VLANs

This module contains actions for viewing & managing VLANs on a Cisco IOS v15 switch.

See the [Overview](overview.md) page for information on how actions in this plugin work.
"""
import logging
import uuid

from typing import List, Optional

from ppa import ui
from ssh_plugin_driver.client import run_list

from .records import CiscoSwitch, VLAN
from ._generic_functions import optional_progress_bar, parse_command_output, check_command_responses, set_terminal_size, session


logger = logging.getLogger(__name__)

TABLE_HEADER = ["Name", "Status", "Interfaces"]


@session
def get_all(
    cisco_switch: CiscoSwitch,
    progress_bar: Optional[bool] = False,
    prompt_timeout: Optional[int] = 5,
    connection_timeout: Optional[int] = 10,
) -> List[VLAN]:
    """Get all VLANs from the switch.

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __cisco_switch:__ a [CiscoSwitch](../inputs_outputs/#ciscoswitch) dictionary

    - __progress_bar:__ set to `true` to display a progress bar when performing the operation (defaults to `false`)

    - __prompt_timeout:__ the number of seconds to wait for commands to complete (defaults to `5`)

    - __connection_timeout:__ the number of seconds to wait for the connection to establish (defaults to `10`)

??? success "Output"
    A list of [VLAN Dictionaries](../inputs_outputs/#vlan).

??? example
    ```YAML linenums="1"
    - cisco_ios_switch.vlans.get_all:
      load:
        cisco_switch: cisco_switch
      save: vlans
    ```
    """
    command = "show vlan brief"
    with optional_progress_bar(
        "Getting all VLANs...", "Got all VLANs", "Failed to get all VLANs", display=progress_bar,
    ):
        output = run_list(
            cisco_switch,
            commands=set_terminal_size([{"command": command}]),
            prompt_timeout=prompt_timeout,
            connection_timeout=connection_timeout,
        )[-1]
        check_command_responses([(command, output)])
        return parse_command_output(command, output)


def input_table(
    text: str,
    vlan_list: List[VLAN],
    expand_ports: Optional[bool] = False,
    exclude_vlans: Optional[List[str]] = None,
    minimum: Optional[int] = None,
    maximum: Optional[int] = None,
) -> List[VLAN]:
    """Display a list of VLANs in a table, and allow the task operator to make a selection.

The table will have the following columns:

- Name
- Status
- Interfaces

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"
    - __text:__ the title of the table

    - __vlan_list:__ a list of [VLAN Dictionaries](../inputs_outputs/#vlan)

    - __exclude_vlans:__ a list of VLAN names to exclude from the table (defaults to `null`)

    - __minimum:__ the minimum number of acceptable selections (defaults to `null`)

    - __maximum:__ the maximum number of acceptable selections (defaults to `null`)

??? success "Output"
    A list of [VLAN Dictionaries](../inputs_outputs/#vlan).


??? example
    - Getting all interfaces from the switch and saving them as variable `all_interfaces`.

    - Displaying all interfaces other than the uplink, and saving the selection as `chosen_interfaces`.

    ```YAML linenums="1"
    - cisco_ios_switch.vlans.get_all:
      load:
        cisco_switch: cisco_switch
      save: all_vlans

    - cisco_ios_switch.vlans.input_table:
        text: Please Select an Interface
        exclude_vlans:
          - Development VLAN
        minimum: 1
        maximum: 1
      load:
        vlan_list: all_vlans
      save: selected_vlan
    ```
    """
    vlan_list = [vlan for vlan in vlan_list if vlan["name"] not in exclude_vlans] if exclude_vlans else vlan_list
    table_element_id = str(uuid.uuid4())
    error_element_id = str(uuid.uuid4())
    while True:
        indices = ui.input_table(
            text,
            header=TABLE_HEADER,
            data=[
                [
                    vlan.get("name") or "",
                    vlan.get("status") or "",
                    ", ".join(vlan.get("interfaces", [])) if expand_ports else len(vlan.get("interfaces", [])),
                ]
                for vlan in vlan_list
            ],
            element_id=table_element_id,
        )
        count = len(indices)
        if minimum:
            if count < minimum:
                ui.output_error(f"Please choose at least {minimum} record(s)", element_id=error_element_id)
                continue
        if maximum:
            if count > maximum:
                ui.output_error(f"Please choose no more than {maximum} record(s)", element_id=error_element_id)
                continue
        break

    return [vlan_list[i] for i in indices]


def output_table(
    text: str, vlan_list: List[VLAN], exclude_vlans: Optional[List[str]] = None, expand_ports: Optional[bool] = False
) -> None:
    """Display a list of VLANs in a table.

The table will have the following columns:

- Name
- Status
- Interfaces

!!! question "Minimum Plugin Version: 1.0.0"


??? info "Input"
    - __text:__ the title of the table

    - __vlan_list:__ a list of [VLAN Dictionaries](../inputs_outputs/#vlan)

    - __exclude_vlans:__ a list of VLAN names to exclude from the table (defaults to `null`)

    - __expand_ports:__ set to `true` to list interface names in the table (defaults to `false`, showing the count instead)


??? success "Output"
    Nothing is outputted by this action.


??? example
    ```YAML linenums="1"
    - cisco_ios_switch.vlans.get_all:
      load:
        cisco_switch: cisco_switch
      save: all_vlans

    - cisco_ios_switch.vlans.output_table:
        text: All VLANs
      load:
        interface_list: all_vlans
    ```
    """
    vlan_list = [vlan for vlan in vlan_list if vlan["name"] not in exclude_vlans] if exclude_vlans else vlan_list
    ui.output_table(
        text,
        header=TABLE_HEADER,
        data=[
            [
                vlan.get("name") or "",
                vlan.get("status") or "",
                ", ".join(vlan.get("interfaces", [])) if expand_ports else len(vlan.get("interfaces", [])),
            ]
            for vlan in vlan_list
        ],
    )
